//----------------------------------------------------------------------
// vx-scheme : Scheme interpreter.  Copyright (c) 2002 Colin Smith.
//
// You may distribute under the terms of the Artistic License, 
// as specified in the LICENSE file.
//
// u-main.cpp : startup code for UNIX or Cygwin environments.

#include <stdio.h>
#include <time.h>
#include "vx-scheme.h"
// The new C++ standard deprecates strstream in favor of stringstream.
// But we would prefer not to let std::string into this code if we can 
// help it, for object-code-size reasons.
#if __GNUG__ >= 3
#include <sstream>
typedef istringstream string_istream;
#else 
#include <strstream>
typedef istrstream string_istream;
#endif
#include <sys/time.h>
#include <unistd.h>
#include <setjmp.h>

static jmp_buf jb;


//----------------------------------------------------------------------------
//
// OS-SPECIFIC FEATURES
//
// This area fills in definitions for OS-specific features named 
// in class OS.  
//

void OS::get_time (int & sec, int & usec)
    {
    struct timeval t;
    gettimeofday (&t, 0);
    sec = t.tv_sec;
    usec = t.tv_usec;
    }

unsigned int OS::flags ()
    {
    static bool env_checked = false;
    static unsigned int f = 0;
    if (! env_checked)
	{
	char * c;
	if ((c = getenv ("T")) != NULL)
	    f = atoi (c);
	env_checked = true;
	}

    return f;
    }

bool OS::interactive (int fd)
    {
    return isatty (fd);
    }

Cell * OS::undef (Context * ctx, const char * name)
    {
    return 0;
    }

void OS::exception (const char * s)
    {
    longjmp (jb, reinterpret_cast <int> (s));
    }

void interact (Context * ctx)
    {
    bool interactive = isatty (0);

    while (ctx->read_eval_print (cin, cout, interactive))
	;
    
    if (OS::flag (DEBUG_MEMSTATS_AT_EXIT))
	ctx->print_mem_stats (cout);

    exit (0);
    }

int main
    (
    int         argc,
    char *      argv []
    )
    {
    const char *    jv;
    Context         ctx;

    // If arguments were specified, treat them as expressions to 
    // evaluate.  Otherwise REPL from standard input.
    if (argc > 1)
	{
	for (int ix = 1; ix < argc; ++ix)
	    {
	    string_istream is (argv [ix]);
	    Cell * result = ctx.eval (ctx.read (is));
	    if (result != unspecified)
		cout << *result << endl;
	    }
	}
    else
	while (1)
	    {
	    if ((jv = reinterpret_cast <const char *> (setjmp (jb))) == 0)
		interact (&ctx);
	    else
		cerr << "caught: " << jv << endl;
	    }
    }

